// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Collected conditions map
///
/// This class is used by the Omnibus controller to manage the conditions of all
/// handlers. The handlers are linked to this class using an index.
class _ConditionMap {
  /// Condition map storage
  Map<String, Map<dynamic, List<int>>> _map;

  /// Constructor
  _ConditionMap() {
    _map = new Map<String, Map<dynamic, List<int>>>();
  }

  /// Insert new index with given conditions.
  insertIndex(int index, Map<String, dynamic> conditions) {
    conditions.forEach((String key, value) {
      _map.putIfAbsent(key, () => new Map<dynamic, List<int>>());
      _map[key].putIfAbsent(value, () => new List<int>());
      _map[key][value].add(index);

      // Make sure the edited List is still sorted.
      _map[key][value].sort((int a, int b) => a - b);
    });
  }

  /// Collect indices for the given conditions.
  Set<int> getIndices(Map<String, dynamic> conditions) {
    assert(conditions != null);

    // Collect index sets for each condition.
    var matrix = new List<List<int>>();
    conditions.forEach((String key, value) {
      if (_map.containsKey(key) && _map[key].containsKey(value)) {
        // Add a copy of the index list under this condition value.
        matrix.add(_map[key][value].toList());
      }
    });

    // Return empty array if the matrix is empty.
    if (matrix.isEmpty) {
      return new Set<int>();
    }

    // Initialize final indices set.
    var indices = new Set<int>();

    // Sort matrix rows from small to large to speed up the index search.
    matrix.sort((List<int> a, List<int> b) => a.length - b.length);

    // Find indices that exist in all rows in the matrix.
    for (var i in matrix[0]) {
      bool foundEverywhere = true;
      for (var j = 1; j < matrix.length; j++) {
        if (matrix[j][0] != i) {
          // This matrix row does not start with $i. This means $i does not
          // exist in this row since it is already sorted in ascending order.
          foundEverywhere = false;
        } else {
          // Remove the first integer in this matrix row (the integer that is
          // equal to $i) so it won't be found a second time.
          matrix[j].removeAt(0);
        }
      }

      // If $i is found in each matrix row it should be included in the
      // response.
      if (foundEverywhere) {
        indices.add(i);
      }
    }

    return indices;
  }
}
