// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.conditions_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages.dart';

main() {
  var bus = new Omnibus();

  test('Bind null handler', () {
    bool caught = false;
    try {
      bus.bindEventHandler(MData.Id, {}, null);
    } on ArgumentError catch (e) {
      caught = e.name == 'handler';
    }
    expect(caught, equals(true));
  });

  test('Publish events while there are no handlers', () {
    // Event with conditions
    bus.publishEvent(new MData({'a': 'a'}));

    // Event without conditions
    bus.publishEvent(new MData({}));
  });

  test('Various conditions filtering', () {
    // Bind some handlers.
    bus.bindEventHandler(
        MData.Id, {'a': 'a', 'b': 'b', 'c': 'c'}, dummyHandler);
    bus.bindEventHandler(
        MData.Id, {'a': 'a', 'b': 'b', 'c': 'a'}, dummyHandler);
    bus.bindEventHandler(
        MData.Id, {'a': 'b', 'b': 'b', 'c': 'a'}, dummyHandler);

    // Publish some events.
    bus.publishEvent(new MData({'a': 'a', 'b': 'b', 'c': 'c'}));
  });

  test('Condition-less handlers', () {
    bus.bindEventHandler(MForEveryone.Id, {}, mForEveryoneHandler);
    bus.publishEvent(new MForEveryone('This message is for everyone!'));
  });
}

Future dummyHandler(MData data) async {
  expect(data.data['a'], equals('a'));
  expect(data.data['b'], equals('b'));
  expect(data.data['c'], equals('c'));
}

Future mForEveryoneHandler(MForEveryone data) async {
  expect(data.message, equals('This message is for everyone!'));
}
