// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.messages;

import 'package:omnibus/omnibus.dart';

/// Email event message
class MEmail extends OmnibusEvent {
  static const String Id = 'omnibus.MEmail';
  String get omnibusId => Id;

  /// Sender
  String from;

  /// Receiver
  String to;

  /// Email HTML body
  String body;

  /// Constructor
  MEmail(this.from, this.to, this.body) {}

  /// Filtering conditions
  Map<String, String> getConditions() {
    return {'from': from, 'to': to};
  }
}

/// Simple calculation types
enum CalcCmd { add, subtract, multiply, divide }

/// Calculation request message
class MCalculationRequest extends OmnibusRequest {
  static const String Id = 'omnibus.MCalculationRequest';
  String get omnibusId => Id;

  /// Input a
  double a;

  /// Input b
  double b;

  /// Calculation type
  CalcCmd calculation;

  /// Constructor
  MCalculationRequest(this.a, this.b, this.calculation);

  /// Filtering conditions
  Map<String, dynamic> getConditions() {
    return {'calculation': calculation};
  }
}

/// Simple calculation response
class MCalculationResponse extends OmnibusResponse {
  static const String Id = 'omnibus.MCalculationResponse';
  String get omnibusId => Id;

  /// Calculation result
  double result;

  /// Constructor
  MCalculationResponse(this.result);
}

/// Univeral event to test conditions.
class MData extends OmnibusEvent {
  static const String Id = 'omnibus.MData';
  String get omnibusId => Id;

  /// Map data
  Map<String, dynamic> data;

  /// Constructor
  MData(this.data);

  /// Filtering conditions
  Map<String, dynamic> getConditions() {
    return data;
  }
}

/// Condition-less event to test conditions.
class MForEveryone extends OmnibusEvent {
  static const String Id = 'omnibus.MForEveryone';
  String get omnibusId => Id;

  /// Message
  String message;

  /// Constructor
  MForEveryone(this.message);
}
