// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.request_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages.dart';

main() {
  test('Run calculations using MCalculationRequest messages', () async {
    var bus = new Omnibus();

    // Bind calculation handlers.
    bus.bindRequestHandler(
        MCalculationRequest.Id, {'calculation': CalcCmd.add}, calcAddHandler);
    bus.bindRequestHandler(MCalculationRequest.Id,
        {'calculation': CalcCmd.subtract}, calcSubtractHandler);
    bus.bindRequestHandler(MCalculationRequest.Id,
        {'calculation': CalcCmd.multiply}, calcMultiplyHandler);
    bus.bindRequestHandler(MCalculationRequest.Id,
        {'calculation': CalcCmd.divide}, calcDivideHandler);

    // Request addition.
    bus
        .publishRequest(new MCalculationRequest(1.0, 1.0, CalcCmd.add))
        .then((AsyncResponses responses) {
      responses.onComplete(() {
        expect(responses.count, equals(1));
        responses.forEach(MCalculationResponse.Id,
            (MCalculationResponse response) {
          expect(response.result, equals(2.0));
        });
      });
    });

    // Request division.
    // This time do not use onComplete but run forEach immediately. In a
    // real-life scenario this can increase the overall performance since the
    // responses that are already collected can be processed while the remaining
    // responses are collected.
    bus
        .publishRequest(new MCalculationRequest(12.0, 8.0, CalcCmd.divide))
        .then((AsyncResponses responses) {
      responses.forEach(MCalculationResponse.Id,
          (MCalculationResponse response) {
        expect(response.result, equals(1.5));
      });
    });
  });
}

Future<MCalculationResponse> calcAddHandler(MCalculationRequest input) async {
  return new MCalculationResponse(input.a + input.b);
}

Future<MCalculationResponse> calcSubtractHandler(
    MCalculationRequest input) async {
  return new MCalculationResponse(input.a - input.b);
}

Future<MCalculationResponse> calcMultiplyHandler(
    MCalculationRequest input) async {
  return new MCalculationResponse(input.a * input.b);
}

Future<MCalculationResponse> calcDivideHandler(
    MCalculationRequest input) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future.delayed(new Duration(seconds: 1),
      () => new MCalculationResponse(input.a / input.b));
}
